<?php
//    Copyright 2010 Andrii Salnikov
//
//   Licensed under the Apache License, Version 2.0 (the "License");
//   you may not use this file except in compliance with the License.
//   You may obtain a copy of the License at
//
//       http://www.apache.org/licenses/LICENSE-2.0
//
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS,
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and
//   limitations under the License.
//

	error_reporting(E_ALL); ini_set("display_errors", 1);
	ini_set("soap.wsdl_cache_enabled", "1");
	ini_set("include_path", "./interfaces:./modules:./lang:./conf");

	require("VOMSSOAP.php");

	// WSDL files
	$wsdl_file = 'wsdl/voms-acl-2.0.2.wsdl';
	$wsdl_server_file = 'wsdl/voms-acl-2.0.2.wsdl';

	// WSDL URL
	$wsdl_url = 'http://'.$_SERVER['HTTP_HOST'].preg_replace('/\?.*/','',$_SERVER['REQUEST_URI']);

	// return WSDL
	if ( isset($_GET['wsdl']) ) {
		$wsdl = file_get_contents($wsdl_file);
		echo preg_replace('/@@ENDPOINT@@/',$wsdl_url,$wsdl);
		exit(0);
	}

	// define handler class
	class VOMSACL extends VOMSSOAP {
		// get acl ID by GID/RID/default
		private function get_aclid ($gid, $rid, $default = 0, $fault = true) {
			$aclid = getACLid ($gid, $rid, $default);
			if ( $fault ) if ( ! $aclid ) $this->request_failed ("Server.userException",
				"NoSuchACLException: ACL for specified Group/Role is not found in VO database");
			return $aclid;
		}

		// get admin ID or create new admin if not exists
		private function get_adminid ($dn, $ca, $create = true) {
			$adminid = getAdminByName($dn, $ca);
			if ( $adminid ) return $adminid;
			if ( $create ) {
				// create if not exists
				$caid = getCAId ( $ca );
				if ( ! $caid ) $this->request_failed ("Server.userException",
					sprintf("NoSuchCAException: There is no such CA '%s' in VO database to create new admin",$ca));
				$acres =  _invoke_transactional_sql(SOAP_UPDATOR_ID,'createAdmin',array('dn'=>$dn,'caid'=>$caid));
				if ( ! $acres ) $this->request_failed ("Server.userException",
					sprintf("CreateAdminException: Cannot create admin %s,%s",$dn,$ca));
				return $GLOBALS['pva_createadmin_insid'];
			} else $this->request_failed ("Server.userException",
				sprintf("NoSuchACLException: There is no ACL entry for specified admin %s,%s",$dn,$ca));
		}

		// return appropriate ACL_Entry type format
		private function acl_entry_t ($dn, $ca, $perm) {
			return 	array(	"adminIssuer" => $ca,
					"adminSubject" => $dn,
					"vomsPermissionBits" => $perm );
		}
		
		public function getACL ($container) {
			// parse container
			$pcont = $this->parse_container($container);
			$group_name = '/' . implode('/',$pcont['groups']);
			// check group name
			$this->parse_group_name($group_name);
			// get group ID
			$gid = $this->get_gid($group_name);
			// check permissions
			$this->acl_check(get_ACL_permissions($gid), "acl", "r" );
			// get role ID
			if ( $pcont['role'] !== NULL ) {
				$rid = $this->get_rid($pcont['role']);
			} else $rid = NULL;
			// get ACL ID
			$aclid = $this->get_aclid($gid, $rid);
			// get ACL permissions
			$acl_perm = getACLvalues($aclid, true);
			$result = array ();
			foreach ( $acl_perm as $ae ) {
				$result[] = $this->acl_entry_t($ae[1], $ae[2], $ae[0]);
			}
			return $result;
		}

		public function getDefaultACL ($group_name) {
			// parse container
			$pcont = $this->parse_container($group_name);
			if ( $pcont['role'] ) $this->request_failed ("Server.userException",
				"IllegalContainerException: Role contexts does not have default ACLs");
			// check group name
			$this->parse_group_name($group_name);
			// get group ID
			$gid = $this->get_gid($group_name);
			// check permissions
			$this->acl_check(get_ACL_permissions($gid), "acl", "r" );
			// get ACL ID
			$aclid = $this->get_aclid($gid, NULL, 1, false);
			if ( ! $aclid ) $this->request_failed ("Server.userException",
				sprintf("NoSuchACLException: Default ACL is not defined for group '%s'", $group_name));
			// get ACL permissions
			$acl_perm = getACLvalues($aclid, true);
			$result = array ();
			foreach ( $acl_perm as $ae ) {
				$result[] = $this->acl_entry_t($ae[1], $ae[2], $ae[0]);
			}
			return $result;
		}

		public function addACLEntry ($container, $acl_entry, $pf) {
			// parse container
			$pcont = $this->parse_container($container);
			$group_name = '/' . implode('/',$pcont['groups']);
			// check group name
			$this->parse_group_name($group_name);
			// get group ID
			$gid = $this->get_gid($group_name);
			// get role ID
			if ( $pcont['role'] !== NULL ) {
				$rid = $this->get_rid($pcont['role']);
			} else $rid = NULL;
			// check permissions
			$this->acl_check(get_ACL_permissions($gid), "acl", "w" );
			// get admin ID
			$admid = $this->get_adminid($acl_entry->adminSubject, $acl_entry->adminIssuer);
			// get ACL ID
			$aclid = $this->get_aclid($gid, $rid, 0, false);
			// create/modify ACL entry
			return _invoke_transactional_sql(SOAP_UPDATOR_ID,'updateACLPermissions',
					$aclid,$admid,$acl_entry->vomsPermissionBits,$pf,$gid,0);
		}

		public function addDefaultACLEntry ($group_name, $acl_entry) {
			// parse container
			$pcont = $this->parse_container($group_name);
			if ( $pcont['role'] ) $this->request_failed ("Server.userException",
				"IllegalContainerException: Role contexts does not have default ACLs");
			// check group name
			$this->parse_group_name($group_name);
			// get group ID
			$gid = $this->get_gid($group_name);
			// check permissions
			$this->acl_check(get_ACL_permissions($gid), "acl", "d" );
			// get admin ID
			$admid = $this->get_adminid($acl_entry->adminSubject, $acl_entry->adminIssuer);
			// get ACL ID
			$aclid = $this->get_aclid($gid, NULL, 1, false);
			// create/modify ACL entry
			return _invoke_transactional_sql(SOAP_UPDATOR_ID,'updateACLPermissions',
					$aclid,$admid,$acl_entry->vomsPermissionBits,0,$gid,1);
		}

		public function removeACLEntry ($container, $acl_entry, $pf) {
			// parse container
			$pcont = $this->parse_container($container);
			$group_name = '/' . implode('/',$pcont['groups']);
			// check group name
			$this->parse_group_name($group_name);
			// get group ID
			$gid = $this->get_gid($group_name);
			// get role ID
			if ( $pcont['role'] !== NULL ) {
				$rid = $this->get_rid($pcont['role']);
			} else $rid = NULL;
			// check permissions
			$this->acl_check(get_ACL_permissions($gid), "acl", "w" );
			// get admin ID
			$admid = $this->get_adminid($acl_entry->adminSubject, $acl_entry->adminIssuer, false);
			// get ACL ID
			$aclid = $this->get_aclid($gid, $rid, 0);
			// delete ACL entry
			return _invoke_transactional_sql(SOAP_UPDATOR_ID,'deleteACLentry',
					$aclid,$admid,$pf,0);
		}

		public function removeDefaultACLEntry ($group_name, $acl_entry) {
			// parse container
			$pcont = $this->parse_container($group_name);
			if ( $pcont['role'] ) $this->request_failed ("Server.userException",
				"IllegalContainerException: Role contexts does not have default ACLs");
			// check group name
			$this->parse_group_name($group_name);
			// get group ID
			$gid = $this->get_gid($group_name);
			// check permissions
			$this->acl_check(get_ACL_permissions($gid), "acl", "d" );
			// get admin ID
			$admid = $this->get_adminid($acl_entry->adminSubject, $acl_entry->adminIssuer, false);
			// get ACL ID
			$aclid = $this->get_aclid($gid, NULL, 1);
			// delete ACL entry
			return _invoke_transactional_sql(SOAP_UPDATOR_ID,'deleteACLentry',
					$aclid,$admid,0,1);
		}

		public function setACL($container,$acl_array) {
			// process acl_array
			$acl_rules = array ();
			foreach ( $acl_array as $acl_entry ) {
				$acl_rules[] =  array ( 'dn' => $acl_entry['adminSubject'],
							'ca' => $acl_entry['adminIssuer'],
							'perm' => $acl_entry['vomsPermissionBits'] );
			}
			// parse container
			$pcont = $this->parse_container($container);
			$group_name = '/' . implode('/',$pcont['groups']);
			// check group name
			$this->parse_group_name($group_name);
			// get group ID
			$gid = $this->get_gid($group_name);
			// get role ID
			if ( $pcont['role'] !== NULL ) {
				$rid = $this->get_rid($pcont['role']);
			} else $rid = NULL;
			// check permissions
			$this->acl_check(get_ACL_permissions($gid), "acl", "w" );
			// get ACL ID
			$aclid = $this->get_aclid($gid, $rid, 0);
			// set ACL entry
			return _invoke_transactional_sql(SOAP_UPDATOR_ID,'setACLPermissions',
					$aclid,$acl_rules,0,array('group_name'=>$group_name));
		}

		public function setDefaultACL($group_name,$acl_array) {
			// process acl_array
			$acl_rules = array ();
			foreach ( $acl_array as $acl_entry ) {
				$acl_rules[] =  array ( 'dn' => $acl_entry['adminSubject'],
							'ca' => $acl_entry['adminIssuer'],
							'perm' => $acl_entry['vomsPermissionBits'] );
			}
			// parse container
			$pcont = $this->parse_container($group_name);
			if ( $pcont['role'] ) $this->request_failed ("Server.userException",
				"IllegalContainerException: Role contexts does not have default ACLs");
			// check group name
			$this->parse_group_name($group_name);
			// get group ID
			$gid = $this->get_gid($group_name);
			// check permissions
			$this->acl_check(get_ACL_permissions($gid), "acl", "d" );
			// get ACL ID
			$aclid = $this->get_aclid($gid, NULL, 1);
			// set ACL entry
			return _invoke_transactional_sql(SOAP_UPDATOR_ID,'setACLPermissions',
					$aclid,$acl_rules,1,array('group_name'=>$group_name));
		}
	}

	// create SAOP server object
	$server = new SoapServer($wsdl_server_file, array(
			'soap_version' => SOAP_1_1 )); 
	$server->setClass('VOMSACL',$wsdl_file);

	// handle client request
	$data = VOMSSOAP::getSOAPRequestData();
	if ( ! $data ) $server->fault('ns1:Client',"Error: method not specified");
	
	// catching SOAPFault exceptions does not work for some reson
	// here the hack to return proper exceptions output
	ob_start();
	$server->handle($data);
	$soapResponse = ob_get_contents();
	ob_end_clean();

	header('HTTP/1.0 200 OK');
	echo $soapResponse;
?>
